<?php

declare(strict_types=1);

namespace Internetgalerie\IgBackendHelpers\RecordList;

use TYPO3\CMS\Core\Database\Query\QueryBuilder;
use TYPO3\CMS\Core\Database\Query\QueryHelper;

class DatabaseRecordList extends \TYPO3\CMS\RecordList\RecordList\DatabaseRecordList
{
    protected $wheres = [];
    /**
     * Initializes the list generation
     *
     * @param int          $id        Page id for which the list is rendered. Must be >= 0
     * @param string       $table     Tablename - if extended mode where only one table is listed at a time.
     * @param int          $pointer   Browsing pointer.
     * @param string       $search    Search word, if any
     * @param int          $levels    Number of levels to search down the page tree
     * @param int          $showLimit Limit of records to be listed.
     * @param array wheres
     */
    public function start($id, $table, $pointer, $search = '', $levels = 0, $showLimit = 0, $wheres=[])
    {
        parent::start($id, $table, $pointer, $search, $levels, $showLimit);
        $this->wheres = $wheres;
    }
    /**
     * Return the modified QueryBuilder object ($queryBuilder) which will be
     * used to select the records from a table $table with pid = $this->pidList
     *
     * @param  string       $table                 Table name
     * @param  int          $pageId                Page id Only used to build the search constraints, $this->pidList is used for restrictions
     * @param array<string> $fieldList List of fields to select from the table
     * @param array<string> $additionalConstraints Additional part for where clause
     */
    public function getQueryBuilder(
        string $table, array $fields = [], bool $addSorting = true, int $firstResult = 0, int $maxResult = 0
    ): QueryBuilder {
        $queryBuilder = parent::getQueryBuilder($table, $pageId, $fields, false, $firstResult, $maxResult);
        if (! empty($this->wheres)) {
            foreach ($this->wheres as $name => $value) {
                if ($value === null) {
                    continue;
                }
                if (! isset($GLOBALS['TCA'][$table]['columns'][$name])) {
                    throw new \InvalidArgumentException('field "'. $name .'" not found in TCA of table "' . $table . '".', 1587402094);
                    continue;
                }

                if (isset($GLOBALS['TCA'][$table]['columns'][$name]['config']['MM'])) {
                    $mm = $GLOBALS['TCA'][$table]['columns'][$name]['config']['MM'];
                    $queryBuilder->leftJoin(
                        $table,
                        $mm,
                        $mm,
                        $queryBuilder->expr()->eq($mm . '.uid_local', $queryBuilder->quoteIdentifier($table . '.uid'))
                    )->andWhere(
                        $queryBuilder->expr()->eq(
                            $mm . '.uid_foreign',
                            $queryBuilder->createNamedParameter($value, \PDO::PARAM_INT)
                        )
                    );
                } else {

                    if (is_array($value)) {
                            $operator = $value['operator'];
                            $parameterValue= $value['value'];
                            $allowedOperators = ['eq', 'neq', 'lt', 'lte', 'gt', 'gte', 'like', 'notLike', 'inSet'];

                            if ($parameterValue != '' && in_array($operator, $allowedOperators)) {
                                if(is_numeric($parameterValue)) {
                                    $queryBuilder->andWhere(
                                        $queryBuilder->expr()->{$operator}(
                                            $name,
                                            $queryBuilder->createNamedParameter($parameterValue, \PDO::PARAM_INT)
                                        )
                                    );
                                } else {
                                    $queryBuilder->andWhere(
                                        $queryBuilder->expr()->{$operator}(
                                            $name,
                                            $queryBuilder->createNamedParameter($parameterValue)
                                        )
                                    );
                                }
                            } else {
                                throw new \InvalidArgumentException('field "'. $name .'" operator not allowed "' . $operator . '" possible values ' . implode(',', $allowedOperators) . '.', 1587402094);
                                 continue;

                            }
                        } else if($value != '') {
                            if(is_numeric($value)) {
                                $queryBuilder->andWhere(
                                    $queryBuilder->expr()->eq(
                                        $name,
                                        $queryBuilder->createNamedParameter($value, \PDO::PARAM_INT)
                                    )
                                );
                            } else {
                                $queryBuilder->andWhere(
                                    $queryBuilder->expr()->eq(
                                        $name,
                                        $queryBuilder->createNamedParameter($value)
                                    )
                                );
                            }
                        }
                }
            }
        }

        if ($addSorting) {
            if ($this->sortField) {
                $queryBuilder->orderBy($table . '.' . $this->sortField, $this->sortRev ? 'DESC' : 'ASC');
            } else {
                $orderBy = $GLOBALS['TCA'][$table]['ctrl']['sortby'] ?: $GLOBALS['TCA'][$table]['ctrl']['default_sortby'];
                $orderBys = QueryHelper::parseOrderBy((string)$orderBy);
                foreach ($orderBys as $orderBy) {
                    $queryBuilder->addOrderBy($table . '.' . $orderBy[0], $orderBy[1]);
                }
            }
        }
        return $queryBuilder;
    }
}
